@@ -27,13 +27,15 @@ __PACKAGE__->mk_accessors(qw[
     key
     key_type
     read_timeout
+    write_timeout
     json
 ]);
 
 my %default = (
-    cert_type    => Net::SSLeay::FILETYPE_PEM(),
-    key_type     => Net::SSLeay::FILETYPE_PEM(),
-    read_timeout => 3,
+    cert_type     => Net::SSLeay::FILETYPE_PEM(),
+    key_type      => Net::SSLeay::FILETYPE_PEM(),
+    read_timeout  => 3,
+    write_timeout => undef,
 );
 
 sub new {
@@ -162,42 +164,57 @@ sub _send {
     my $self = shift;
     my $data = \$_[0];
     my ($sock, $ctx, $ssl) = @{$self->_connect};
-    Net::SSLeay::ssl_write_all($ssl, $data) or _die_if_ssl_error("ssl_write_all error: $!");
 
+    return unless $self->_do_select($sock, 'write', $self->write_timeout);
+
+    Net::SSLeay::ssl_write_all($ssl, $data) or _die_if_ssl_error("ssl_write_all error: $!");
     return 1;
 }
 
 sub _read {
     my $self = shift;
-
-    my $begin_time = Time::HiRes::time();
-    my $timeout = $self->read_timeout;
-
     my ($sock, $ctx, $ssl) = @{$self->_connect};
 
-    my $data;
+    return unless $self->_do_select($sock, 'read', $self->read_timeout);
+
+    my $data = Net::SSLeay::ssl_read_all($ssl) or _die_if_ssl_error("ssl_read_all error: $!");
+    return $data;
+}
+
+sub _do_select {
+    my ($self, $sock, $act, $timeout) = @_;
+
+    my $begin_time = Time::HiRes::time();
+
+    vec(my $bits = '', fileno($sock), 1) = 1;
     while (1) {
-        vec(my $rin = '', fileno($sock), 1) = 1;
-        my $nfound = select($rin, undef, undef, $timeout);
+        my $nfound;
+        if ($act eq 'read') {
+            $nfound = select my $rout = $bits, undef, undef, $timeout;
+        }
+        else {
+            $nfound = select undef, my $wout = $bits, undef, $timeout;
+        }
         return unless $nfound; # timeout
 
         # returned error
         if ($nfound == -1) {
             if ($! == EINTR) {
                 # can retry
-                $timeout -= ($begin_time - Time::HiRes::time());
+                $timeout -= (Time::HiRes::time() - $begin_time) if defined $timeout;
                 next;
             }
-
-            $self->disconnect;
-            return;
+            else {
+                # other error
+                $self->disconnect;
+                return;
+            }
         }
 
-        $data = Net::SSLeay::ssl_read_all($ssl) or _die_if_ssl_error("ssl_read_all error: $!");
         last;
     }
 
-    return $data;
+    return 1;
 }
 
 sub DESTROY {
